@@ -489,13 +489,13 @@ end
     @test B == A .* A'
 end
 
-@testset "test show methods for $t of Factorizations" for t in (Adjoint, Transpose)
-    A = randn(4, 4)
+@testset "test show methods for $t of Factorizations" for t in (adjoint, transpose)
+    A = randn(ComplexF64, 4, 4)
     F = lu(A)
     Fop = t(F)
-    @test "LinearAlgebra."*sprint(show, Fop) ==
+    @test sprint(show, Fop) ==
                   "$t of "*sprint(show, parent(Fop))
-    @test "LinearAlgebra."*sprint((io, t) -> show(io, MIME"text/plain"(), t), Fop) ==
+    @test sprint((io, t) -> show(io, MIME"text/plain"(), t), Fop) ==
                   "$t of "*sprint((io, t) -> show(io, MIME"text/plain"(), t), parent(Fop))
 end
 
@@ -636,4 +636,49 @@ end
     @test mapreduce(string, *, [1 2; 3 4]') == mapreduce(string, *, copy([1 2; 3 4]')) == "1234"
 end
 
+@testset "trace" begin
+    for T in (Float64, ComplexF64), t in (adjoint, transpose)
+        A = randn(T, 10, 10)
+        @test tr(t(A)) ≈ tr(copy(t(A))) ≈ t(tr(A))
+    end
+end
+
+@testset "structured printing" begin
+    D = Diagonal(1:3)
+    @test sprint(Base.print_matrix, Adjoint(D)) == sprint(Base.print_matrix, D)
+    @test sprint(Base.print_matrix, Transpose(D)) == sprint(Base.print_matrix, D)
+    D = Diagonal((1:3)*im)
+    D2 = Diagonal((1:3)*(-im))
+    @test sprint(Base.print_matrix, Transpose(D)) == sprint(Base.print_matrix, D)
+    @test sprint(Base.print_matrix, Adjoint(D)) == sprint(Base.print_matrix, D2)
+
+    struct OneHotVecOrMat{N} <: AbstractArray{Bool,N}
+        inds::NTuple{N,Int}
+        sz::NTuple{N,Int}
+    end
+    Base.size(x::OneHotVecOrMat) = x.sz
+    function Base.getindex(x::OneHotVecOrMat{N}, inds::Vararg{Int,N}) where {N}
+        checkbounds(x, inds...)
+        inds == x.inds
+    end
+    Base.replace_in_print_matrix(o::OneHotVecOrMat{1}, i::Integer, j::Integer, s::AbstractString) =
+        o.inds == (i,) ? s : Base.replace_with_centered_mark(s)
+    Base.replace_in_print_matrix(o::OneHotVecOrMat{2}, i::Integer, j::Integer, s::AbstractString) =
+        o.inds == (i,j) ? s : Base.replace_with_centered_mark(s)
+
+    o = OneHotVecOrMat((2,), (4,))
+    @test sprint(Base.print_matrix, Transpose(o)) == sprint(Base.print_matrix, OneHotVecOrMat((1,2), (1,4)))
+    @test sprint(Base.print_matrix, Adjoint(o)) == sprint(Base.print_matrix, OneHotVecOrMat((1,2), (1,4)))
+end
+
+@testset "error message in transpose" begin
+    v = zeros(2)
+    A = zeros(1,1)
+    B = zeros(2,3)
+    for (t1, t2) in Any[(A, v), (v, A), (A, B)]
+        @test_throws "axes of the destination are incompatible with that of the source" transpose!(t1, t2)
+        @test_throws "axes of the destination are incompatible with that of the source" adjoint!(t1, t2)
+    end
+end
+
 end # module TestAdjointTranspose
