@@ -2,6 +2,7 @@ module Artifacts
 
 using Artifacts, Base.BinaryPlatforms, SHA
 using ..MiniProgressBars, ..PlatformEngines
+using Tar: can_symlink
 
 import ..set_readonly, ..GitTools, ..TOML, ..pkg_server, ..can_fancyprint,
        ..stderr_f, ..printpkgstyle
@@ -48,12 +49,7 @@ function create_artifact(f::Function)
         # as something that was foolishly overridden.  This should be virtually impossible
         # unless the user has been very unwise, but let's be cautious.
         new_path = artifact_path(artifact_hash; honor_overrides=false)
-        if !isdir(new_path)
-            # Move this generated directory to its final destination, set it to read-only
-            mv(temp_dir, new_path)
-            chmod(new_path, filemode(dirname(new_path)))
-            set_readonly(new_path)
-        end
+        _mv_temp_artifact_dir(temp_dir, new_path)
 
         # Give the people what they want
         return artifact_hash
@@ -63,6 +59,48 @@ function create_artifact(f::Function)
     end
 end
 
+"""
+    _mv_temp_artifact_dir(temp_dir::String, new_path::String)::Nothing
+Either rename the directory at `temp_dir` to `new_path` and set it to read-only
+or if `new_path` artifact already exists try to do nothing.
+"""
+function _mv_temp_artifact_dir(temp_dir::String, new_path::String)::Nothing
+    # Sometimes a rename can fail because the temp_dir is locked by
+    # anti-virus software scanning the new files.
+    # In this case we want to sleep and try again.
+    # I am using the list of error codes to retry from:
+    # https://github.com/isaacs/node-graceful-fs/blob/234379906b7d2f4c9cfeb412d2516f42b0fb4953/polyfills.js#L87
+    # Retry for up to about 60 seconds by retrying 20 times with exponential backoff.
+    retry = 0
+    max_num_retries = 20 # maybe this should be configurable?
+    sleep_amount = 0.01 # seconds
+    max_sleep_amount = 5.0 # seconds
+    while true
+        isdir(new_path) && return
+        # This next step is like
+        # `mv(temp_dir, new_path)`.
+        # However, `mv` defaults to `cp` if `rename` returns an error.
+        # `cp` is not atomic, so avoid the potential of calling it.
+        err = ccall(:jl_fs_rename, Int32, (Cstring, Cstring), temp_dir, new_path)
+        if err ≥ 0
+            # rename worked
+            chmod(new_path, filemode(dirname(new_path)))
+            set_readonly(new_path)
+            return
+        else
+            # Ignore rename error if `new_path` exists.
+            isdir(new_path) && return
+            if retry < max_num_retries && err ∈ (Base.UV_EACCES, Base.UV_EPERM, Base.UV_EBUSY)
+                sleep(sleep_amount)
+                sleep_amount = min(sleep_amount*2.0, max_sleep_amount)
+                retry += 1
+            else
+                Base.uv_error("rename of $(repr(temp_dir)) to $(repr(new_path))", err)
+            end
+        end
+    end
+end
+
 """
     remove_artifact(hash::SHA1; honor_overrides::Bool=false)
 
@@ -288,18 +326,64 @@ function download_artifact(
         return true
     end
 
-    # We download by using `create_artifact()`.  We do this because the download may
+    # Ensure the `artifacts` directory exists in our default depot
+    artifacts_dir = first(artifacts_dirs())
+    mkpath(artifacts_dir)
+    # expected artifact path
+    dst = joinpath(artifacts_dir, bytes2hex(tree_hash.bytes))
+
+    # We download by using a temporary directory.  We do this because the download may
     # be corrupted or even malicious; we don't want to clobber someone else's artifact
     # by trusting the tree hash that has been given to us; we will instead download it
     # to a temporary directory, calculate the true tree hash, then move it to the proper
     # location only after knowing what it is, and if something goes wrong in the process,
-    # everything should be cleaned up.  Luckily, that is precisely what our
-    # `create_artifact()` wrapper does, so we use that here.
-    calc_hash = try
-        create_artifact() do dir
-            download_verify_unpack(tarball_url, tarball_hash, dir, ignore_existence=true, verbose=verbose,
+    # everything should be cleaned up.
+
+    # Temporary directory where we'll do our creation business
+    temp_dir = mktempdir(artifacts_dir)
+
+    try
+        download_verify_unpack(tarball_url, tarball_hash, temp_dir, ignore_existence=true, verbose=verbose,
                 quiet_download=quiet_download, io=io)
+        calc_hash = SHA1(GitTools.tree_hash(temp_dir))
+
+        # Did we get what we expected?  If not, freak out.
+        if calc_hash.bytes != tree_hash.bytes
+            msg = """
+            Tree Hash Mismatch!
+              Expected git-tree-sha1:   $(bytes2hex(tree_hash.bytes))
+              Calculated git-tree-sha1: $(bytes2hex(calc_hash.bytes))
+            """
+            # Since tree hash calculation is rather fragile and file system dependent,
+            # we allow setting JULIA_PKG_IGNORE_HASHES=1 to ignore the error and move
+            # the artifact to the expected location and return true
+            ignore_hash_env_set = get(ENV, "JULIA_PKG_IGNORE_HASHES", "") != ""
+            if ignore_hash_env_set
+                ignore_hash = Base.get_bool_env("JULIA_PKG_IGNORE_HASHES", false)
+                ignore_hash === nothing && @error(
+                    "Invalid ENV[\"JULIA_PKG_IGNORE_HASHES\"] value",
+                    ENV["JULIA_PKG_IGNORE_HASHES"],
+                )
+                ignore_hash = something(ignore_hash, false)
+            else
+                # default: false except Windows users who can't symlink
+                ignore_hash = Sys.iswindows() &&
+                    !mktempdir(can_symlink, artifacts_dir)
+            end
+            if ignore_hash
+                desc = ignore_hash_env_set ?
+                    "Environment variable \$JULIA_PKG_IGNORE_HASHES is true" :
+                    "System is Windows and user cannot create symlinks"
+                msg *= "\n$desc: \
+                    ignoring hash mismatch and moving \
+                    artifact to the expected location"
+                @error(msg)
+            else
+                error(msg)
+            end
         end
+        # Move it to the location we expected
+        _mv_temp_artifact_dir(temp_dir, dst)
     catch err
         @debug "download_artifact error" tree_hash tarball_url tarball_hash err
         if isa(err, InterruptException)
@@ -307,29 +391,15 @@ function download_artifact(
         end
         # If something went wrong during download, return the error
         return err
-    end
-
-    # Did we get what we expected?  If not, freak out.
-    if calc_hash.bytes != tree_hash.bytes
-        msg  = "Tree Hash Mismatch!\n"
-        msg *= "  Expected git-tree-sha1:   $(bytes2hex(tree_hash.bytes))\n"
-        msg *= "  Calculated git-tree-sha1: $(bytes2hex(calc_hash.bytes))"
-        # Since tree hash calculation is still broken on some systems, e.g. Pkg.jl#1860,
-        # and Pkg.jl#2317, we allow setting JULIA_PKG_IGNORE_HASHES=1 to ignore the
-        # error and move the artifact to the expected location and return true
-        ignore_hash = Base.get_bool_env("JULIA_PKG_IGNORE_HASHES", false)
-        if ignore_hash
-            msg *= "\n\$JULIA_PKG_IGNORE_HASHES is set to 1: ignoring error and moving artifact to the expected location"
-            @error(msg)
-            # Move it to the location we expected
-            src = artifact_path(calc_hash; honor_overrides=false)
-            dst = artifact_path(tree_hash; honor_overrides=false)
-            mv(src, dst; force=true)
-            return true
+    finally
+        # Always attempt to cleanup
+        try
+            rm(temp_dir; recursive=true, force=true)
+        catch e
+            e isa InterruptException && rethrow()
+            @warn("Failed to clean up temporary directory $(repr(temp_dir))", exception=e)
         end
-        return ErrorException(msg)
     end
-
     return true
 end
 
@@ -431,13 +501,20 @@ function with_show_download_info(f, io, name, quiet_download)
         fancyprint && print_progress_bottom(io)
         printpkgstyle(io, :Downloading, "artifact: $name")
     end
+    success = false
     try
-        return f()
+        result = f()
+        success = result === true
+        return result
     finally
         if !quiet_download
             fancyprint && print(io, "\033[1A") # move cursor up one line
             fancyprint && print(io, "\033[2K") # clear line
-            fancyprint && printpkgstyle(io, :Downloaded, "artifact: $name")
+            if success
+                fancyprint && printpkgstyle(io, :Downloaded, "artifact: $name")
+            else
+                printpkgstyle(io, :Failure, "artifact: $name", color = :red)
+            end
         end
     end
 end
