@@ -303,6 +303,7 @@ PSA[:name => "activate",
     activate
     activate [--shared] path
     activate --temp
+    activate - (activates the previously active environment)
 
 Activate the environment at the given `path`, or use the first project found in `LOAD_PATH` if no `path` is specified.
 The active environment is the environment that is modified by executing package commands.
@@ -311,6 +312,7 @@ When the option `--shared` is given, `path` will be assumed to be a directory na
 it will be placed in the first depot of the stack.
 Use the `--temp` option to create temporary environments which are removed when the julia
 process is exited.
+Use a single `-` to activate the previously active environment.
 """ ,
 ],
 PSA[:name => "update",
@@ -362,6 +364,7 @@ Create a minimal project called `pkgname` in the current folder. For more featur
 ],
 PSA[:name => "precompile",
     :api => API.precompile,
+    :should_splat => false,
     :arg_count => 0 => Inf,
     :completions => complete_installed_packages,
     :description => "precompile all the project dependencies",
